import sys
from os.path import isdir

ipt='/sbin/iptables'
prerouting_target='-t nat -A PREROUTING'
dport_target='--dport'
forward_target='-A FORWARD'
mangle_target='-t mangle -A FORWARD'
tcpmss_target='-j TCPMSS --set-mss'
tcp_syn_port_target='-p tcp --syn'
tcp_port_target='{0} {1}'.format(tcp_syn_port_target, dport_target)
udp_port_target='-p udp -m udp {0}'.format(dport_target)
icmp_target='-p icmp'
icmp_type_target='--icmp-type'
esp_target='-p 50'
dnat_target='-j DNAT --to-destination'
accept_target='-j ACCEPT'
state_new='-m state --state NEW'
state_policy='-m policy'
state_ipsec_target='{0} --pol ipsec --proto esp --dir'.format(state_policy)

def join(*args):
    return " ".join(list(args))

def test_mode(mode, value):
    if type(mode) == list:
        if value in mode:
            return True
        return False
    else:
        if mode == value:
            return True
        return False

class Rule:
    def __init__(self, ip1, ip2, dest_ip, port1, mimetype, protocol='tcp',
            dest=True, filter_type='allow', interface1='eth0', port2=None,
            interface2='eth0', bridge='br0', typ=None, mode='state_new', value=None):
        """
        dest: True: container is the source, extremity is the destination
        """
        self.ip1 = ip1
        self.ip2 = ip2
        self.dest_ip = dest_ip
        self.port1 = port1
        self.port2 = port2
        self.interface1 = interface1
        self.interface2 = interface2
        self.filter_type = filter_type
        self.dest = dest
        self.mimetype = mimetype
        self.protocol = protocol
        self.typ = typ
        self.mode = mode
        self.value = value
        #if self.mimetype == 'wide' and dest == True:
        #    sys.stderr.write("Error: container cannot access in WAN.\nRule: %s - %s:%s not allowed\n"%(
        #        ip1, ip2, port))
        #    sys.exit(1)
        self.comment = False
        if str(ip2) == "127.0.0.1" or str(ip1) == str(ip2):
            self.comment = "# rule from %s to %s port1 %s int %s only if container mode is activated" % (ip1, ip2, port1, interface1)

    def is_era(self):
        return isdir('/usr/share/era')

    def _build_wide_src_allow(self):
        if self.is_era():
            return "# rule from %s to %s port1 %s int %s desactivated with era" % (self.ip1, self.ip2, self.port1, self.interface1)
        if self.protocol == 'tcp':
            port_target = tcp_port_target
        elif self.protocol == 'udp':
            port_target = udp_port_target
        else:
            raise Exception("Protocol %s not supported in _build_wide_src_allow"%self.protocol)

        ret = join(ipt, forward_target, '-i', self.interface1, '-s',
                str(self.ip2), port_target, str(self.port1),
                '-d', str(self.ip1), accept_target)
        ret += '\n'
        ret += join(ipt, prerouting_target, '-i', self.interface1, '-s',
                str(self.ip2), port_target, str(self.port2),
                dnat_target, '{0}:{1}'.format(str(self.ip1), str(self.port2)))
        return ret

    def _build_wide_dst_allow(self):
        #if self.is_era():
        #    return "# rule from %s to %s port %s int %s desactivated with era" % (self.ip1, self.ip2, self.port, self.interface)
        if self.protocol == 'tcp':
            build_target = join(tcp_port_target, str(self.port1))
        elif self.protocol == 'udp':
            build_target = join(udp_port_target, str(self.port1))
        elif self.protocol == 'icmp':
            build_target = icmp_target
            if self.typ != None:
                build_target = join(build_target, icmp_type_target, self.typ)
        else:
            raise Exception("Protocol %s not supported in _build_wide_dst_allow"%self.protocol)

        return join(ipt, forward_target, '-i', 'br0', '-o', self.interface1,
                '-s', str(self.ip1), build_target,
                '-d', str(self.ip2), accept_target)

    def _build_container_dst_allow(self):
        #interface not need, but must be specified if container are disabled
        if self.interface2 != 'eth0' and self.interface2 != None:
            return "# rule from %s to %s port1 %s with container interface %s desactivated" % (self.ip2, self.ip1, self.port1, self.interface2)
        if self.protocol == 'tcp':
            port_target = tcp_port_target
        elif self.protocol == 'udp':
            port_target = udp_port_target
        else:
            raise Exception("Protocol %s not supported in _build_container_dst_allow"%self.protocol)

        return join(ipt, forward_target, '-i', 'br0', '-s',
                str(self.ip1), port_target, str(self.port1),
                '-d', str(self.ip2), accept_target)

    def _build_bridge_dst_allow(self):
        #interface not need, but must be specified if container are disabled
        if self.interface2 != 'eth0' and self.interface2 != None:
            return "# rule from %s to %s port1 %s with container interface %s desactivated" % (self.ip2, self.ip1, self.port1, self.interface2)
        if self.protocol == 'tcp':
            port_target = tcp_port_target
        elif self.protocol == 'udp':
            port_target = udp_port_target
        else:
            raise Exception("Protocol %s not supported in _build_bridge_dst_allow"%self.protocol)

        return join(ipt, '-A', 'cont-root', '-i', 'br0', '-s',
                str(self.ip1), port_target, str(self.port1),
                '-d', str(self.ip2), accept_target)

    def _get_mode(self, mode):
        if None not in [self.interface1, self.interface2] and self.interface1 != self.interface2:
            if self.interface1 == 'eth0':
                level='in'
            else:
                level='out'
        else:
            level='in'
        if mode == None:
            return ''
        if mode == 'state_new':
            return state_new
        elif mode == 'policy_ipsec':
            return join(state_ipsec_target, level)
        elif mode == 'mangle':
            return join(mangle_target, tcp_syn_port_target)
        else:
            raise Exception('Unknown mode {0}'.format(mode))

    def _build_root_dst_allow(self):
        if self.is_era():
            return "# rule from %s to %s port1 %s int %s desactivated with era" % (self.ip1, self.ip2, self.port1, self.interface1)
        if type(self.mode) == list:
            mode = ''
            for mod in self.mode:
                mode = join(mode, self._get_mode(mod))
        else:
            mode = self._get_mode(self.mode)

        if self.protocol == 'tcp':
            build_target = join(state_new, tcp_port_target, str(self.port1))
        elif self.protocol == 'udp':
            build_target = join(udp_port_target, str(self.port1))
        elif self.protocol == 'icmp':
            build_target = icmp_target
            if self.typ != None:
                build_target = join(build_target, icmp_type_target, self.typ)
        elif self.protocol == 'esp':
            build_target = esp_target
        elif self.protocol == None:
            if self.port1 != None:
                build_target = join(dport_target, self.port1)
            else:
                build_target = ''
        else:
            raise Exception("Protocol %s not supported in _build_root_dst_allow"%self.protocol)
        if str(self.ip2) != 'None':
            source_target = join('-s', str(self.ip2))
        else:
            source_target = ''

        if None not in [self.interface1, self.interface2] and self.interface1 != self.interface2:
            name_target = '{0}-{1}'.format(self.interface1, self.interface2)
        elif self.interface1 == 'eth0':
            name_target = 'wide-root'
        else:
            name_target = '{0}-root'.format(self.interface1)
        return join(ipt, '-A', name_target, mode, build_target, source_target, accept_target)

    def _build_local_dst_allow(self):
        if self.protocol == 'tcp':
            port_target = tcp_port_target
        elif self.protocol == 'udp':
            port_target = udp_port_target
        else:
            raise Exception("Protocol %s not supported in _build_container_dst_allow"%self.protocol)

        ret = join(ipt, forward_target, '-i', 'br0', '-s',
                str(self.ip1), port_target, str(self.port1),
                '-d', str(self.ip2), accept_target)
        ret += '\n'
        ret += join(ipt, '-A', 'INPUT', '-i', 'br0', '-s',
                str(self.ip1), port_target, str(self.port1),
                '-d', str(self.ip2), accept_target)
        return ret

    def _build_widewide_dst_allow(self):
        if self.is_era():
            return "# rule from %s to %s port1 %s int %s desactivated with era" % (self.ip1, self.ip2, self.port1, self.interface1)
        if type(self.mode) == list:
            mode = ''
            for mod in self.mode:
                mode = join(mode, self._get_mode(mod))
        else:
            mode = self._get_mode(self.mode)

        build_target = ''
        end_target = join('-s', str(self.ip2), accept_target)
        append_target = join('-A', 'wide-wide')
        if self.protocol == 'tcp':
            build_target = join(tcp_port_target, str(self.port1))
        elif self.protocol == 'udp':
            build_target = join(udp_port_target, str(self.port1))
        elif self.protocol == 'icmp':
            build_target = icmp_target
            if self.typ != None:
                build_target = join(build_target, icmp_type_target, self.typ)
        elif self.protocol == 'esp':
            build_target = esp_target
        if test_mode(self.mode, 'mangle') and self.typ == 'tcpmss':
            build_target = join(tcpmss_target, self.value)
            end_target = ''
            append_target = ''
        elif self.protocol == None:
            pass
        else:
            raise Exception("Protocol %s not supported in _build_root_dst_allow"%self.protocol)
        if self.port1 != None:
            build_target = join(build_target, dport_target, self.port1)
        if str(self.dest_ip) != str(None):
            build_target = join(build_target, '-d', str(self.dest_ip))

        return join(ipt, append_target, mode, build_target, end_target)

    def _build_root_src_allow(self):
        if self.is_era():
            return "# rule from %s to %s port %s int %s desactivated with era" % (self.ip2, self.ip1, self.port, self.interface1)
        if self.protocol == 'tcp':
            port_target = tcp_port_target
        elif self.protocol == 'udp':
            port_target = udp_port_target
        else:
            raise Exception("Protocol %s not supported in _build_root_src_allow"%self.protocol)
        return join(ipt, '-A', 'wide-root', state_new,
                '-s', str(self.ip2), port_target, str(self.port1),
                accept_target)

    def __str__(self):
        try:
            if self.comment != False:
                return self.comment
            if self.dest:
                dest='dst'
            else:
                dest='src'

            return getattr(self, '_build_'+self.mimetype+'_'+dest+'_'+self.filter_type)()
        except AttributeError:
            #import traceback
            #print traceback.print_exc()
            #print '_build_'+self.mimetype+'_'+dest+'_'+self.filter_type
            if self.dest:
                return "# rule from %s to %s port1 %s int %s not implemented" % (self.ip1, self.ip2, self.port1, self.interface1)
            else:
                return "# rule from %s to %s port1 %s int %s not implemented" % (self.ip2, self.ip1, self.port1, self.interface1)

