from os.path import join, basename, splitext, isfile
from glob import glob
from IPy import IP
import sys

from creole.config import firewall_rules_dir
from bastion.fwobjects import Rule
from bastion.twobjects import TcpWrapper
from creole.eosfunc import load_container_var
from creole import parsedico

dicoEole = parsedico.parse_dico()

global rules
global tcpwrappers
global fname
rules = []
tcpwrappers = []

network_containers = load_container_var()
adresse_ip_br0 = IP(network_containers['adresse_ip_br0'])
if adresse_ip_br0 != IP('127.0.0.1'):
    network = network_containers.get('adresse_network_br0', None)
    if network == None:
        sys.stderr.write('Network introuvable\n')
        sys.exit(1)
    subnetwork = IP('%s/24'%network)

local_ip = []
for num in range(int(dicoEole['nombre_interfaces'])):
    local_ip.append(dicoEole['adresse_ip_eth{0}'.format(num)])

def test_mode(mode, value):
    if type(mode) == list:
        if value in mode:
            return True
        return False
    else:
        if mode == value:
            return True
        return False

def test_port(ports, protocol, mode):
    def test():
        for port in ports.split(':'):
            try:
                port = int(port)
            except ValueError, e:
                sys.stderr.write('Error port: %s is not an integer: %s in %s\n'%(str(port), str(e), fname))
                sys.exit(1)
            if port not in range(0, 49151):
                sys.stderr.write('Error port not a valid port: %s in %s' % (ports, fname))
    if protocol in ['icmp', 'esp'] or mode == 'mangle':
        if ports != None:
            raise Exception('Error port: no port for %s protocol'%protocol)
    elif test_mode(mode, 'policy_ipsec'):
        if ports != None:
            test()
    else:
        if ports == None:
            raise Exception('Error port: no port specified')
        test()

def test_typ(typ, protocol, mode):
    if protocol != 'icmp' and not test_mode(mode, 'mangle'):
        if typ != None:
            raise Exception('Error typ: only for ICMP rule')
    if (protocol == 'icmp' and typ not in [None, 'echo-request']) or (test_mode(mode, 'mangle') and typ != 'tcpmss'):
        raise Exception('Error typ: not a valid typ: %s'%str(typ))

def get_mimetype(container_ip, extr_ip, dest_ip):
    if container_ip == IP("127.0.0.1"):
        if str(dest_ip) != str(None):
            return 'widewide'
        return 'root'
    else:
        if extr_ip == adresse_ip_br0:
            return 'bridge'
        elif extr_ip in subnetwork:
            return 'container'
        elif str(extr_ip) in local_ip:
            return 'local'
        else:
            return 'wide'

def allow(container, ip, dest_ip, port, dest_port, interface, dest, tcpwrapper,
             interface2, protocol, typ, mode, value):
    mimetype = None
    if mode != 'policy_ipsec' and (None not in [container, ip] or container == ip ):
        sys.stderr.write('Need container OR ip')
        sys.exit(1)
    if protocol not in ['tcp', 'udp', 'icmp', 'esp', None]:
        sys.stderr.write('Unkown protocol %s'%protocol)
    if dest_ip != None:
        try:
            dest_ip = IP(dest_ip)
        except Exception, e:
            sys.stderr.write('Error for dest_ip %s: %s in %s\n'%(str(dest_ip), str(e), fname))
            sys.exit(1)
    if ip != None:
        try:
            ip = IP(ip)
        except Exception, e:
            mimetype = 'wide'

    elif container != None:
        if container == 'root':
            ip = adresse_ip_br0
        else:
            global network_containers
            if network_containers.has_key('container_ip_%s' % container):
                ip = IP(network_containers['container_ip_%s' % container])
            else:
                sys.stderr.write('Container %s not already exists'%container)
                sys.exit(1)

    test_port(port, protocol, mode)
    test_typ(typ, protocol, mode)

    cip = IP(container_ip)
    if mimetype == None:
        mimetype = get_mimetype(cip, ip, dest_ip)
    global rules, tcpwrappers
    if tcpwrapper != None and mimetype == 'root':
        if tcpwrapper == '':
            sys.stderr.write('No name for tcpwrapper')
        tcpwrappers.append(TcpWrapper(name=tcpwrapper, ip=ip))
    if dest_port != None:
        if mimetype != 'wide' or dest:
            raise Exception('dest_port only if wide address')
        test_port(dest_port, protocol, mode)
    else:
        dest_port = port
    rules.append(Rule(ip1=cip, ip2=ip, dest_ip=dest_ip, port1=port,
        port2=dest_port, interface1=interface, dest=dest, mimetype=mimetype,
        interface2=interface2, protocol=protocol, typ=typ, mode=mode, value=value))

def allow_dest(container=None, ip=None, port=None, dest_port=None, tcpwrapper=None,
            interface='eth0', interface2=None, container_interface=None, protocol='tcp', typ=None):
    if None not in [interface2, container_interface]:
        raise Exception('interface2 not allow with container_interface')
    if container_interface != None:
        interface2 = container_interface
    allow(container=container, ip=ip, dest_ip=None, port=port, dest=True, dest_port=dest_port, tcpwrapper=tcpwrapper,
            interface=interface, interface2=interface2, protocol=protocol, typ=typ, mode=None, value=None)

def allow_src(container=None, ip=None, dest_ip=None, port=None, dest_port=None, tcpwrapper=None,
            interface='eth0', interface2=None, container_interface=None, protocol='tcp', typ=None):
    dest=False
    if None not in [interface2, container_interface]:
        raise Exception('interface2 not allow with container_interface')
    if container_interface != None:
        interface2 = container_interface
    allow(container=container, ip=ip, dest_ip=dest_ip, port=port, dest_port=dest_port, dest=False, tcpwrapper=tcpwrapper,
            interface=interface, interface2=interface2, protocol=protocol, typ=typ, mode=None, value=None)

def allow_ipsec(ip=None, dest_ip=None, port=None, dest_port=None, mode='policy_ipsec',
            interface=None, interface2=None, protocol=None):
    allow(container=None, ip=ip, dest_ip=dest_ip, port=port, dest=True, dest_port=dest_port, tcpwrapper=None,
            interface=interface, interface2=interface2, protocol=protocol, typ=None, mode=mode, value=None)

def mangle(interface, interface2, typ, value):
    mode = 'mangle'
    protocol = None
    test_typ(typ, protocol, mode)
    cip = IP(container_ip)
    if str(cip) != '127.0.0.1':
        raise Exception('mangle must be used only for container root')
    mimetype = 'widewide'
    global rules
    #seulement pour faire passer le test
    ip2="0.0.0.0/0"
    rules.append(Rule(ip1=None, ip2=ip2, dest_ip=None, port1=None,
        port2=None, interface1=interface, dest=True, mimetype=mimetype,
        interface2=interface2, protocol=protocol, typ=typ, mode=mode, value=value))

def file_loader():
    files = glob(join(firewall_rules_dir, '*_*.fw'))
    files.sort()
    global network_containers
    global fname
    for fname in files:
        # puts name in global namespace because we need it in execfile's
        # namespace in rules_loader
        global container_ip
        name = splitext(basename(fname))[0].split('_')[1]
        if network_containers.has_key('container_ip_%s' % name):
            container_ip = network_containers['container_ip_%s' % name]
        else:
            raise NameError('Container %s not already exists'%name)
        # load rules from file with exec to avoid encoding errors
        exec(file(fname).read())

    return (rules, tcpwrappers)

if __name__ == '__main__':
    firewall, tcpwrapper = file_loader()
    print "------iptables------"
    for rule in firewall:
        print rule
    print "------tcpwrapper------"
    for tcpwrap in tcpwrapper:
        print tcpwrap
